@@ -5,14 +5,13 @@
 #include <string.h>
 #include "pdflib.h"
 #include "wishart.h"
-#include "rnglib.h"
 #include<R.h>
 #include<Rinternals.h>
 #include<Rmath.h>
 
 SEXP jomo1ranmix(SEXP Y, SEXP Yimp, SEXP Yimp2, SEXP Yimpcat, SEXP X, SEXP Z, SEXP clus, SEXP beta, SEXP u, SEXP betapost, SEXP upost, SEXP omega, SEXP omegapost, SEXP covu, SEXP covupost, SEXP nstep, SEXP Sp, SEXP Sup, SEXP Y_numcat, SEXP num_con, SEXP flagrng){
 int indic=0,i,j,k, IY,JY, IX, JX, Io, Jo, Ib, Jb, ns, nmiss=0,t, countm=0, counto=0, countmm=0, countmo=0,countoo=0, jj, tt, kk, ncon,ncat, pos,flag=0,nmaxx,h;
-int Iu, Ju, IZ, JZ, nj,c,fl;
+int Iu, Ju, IZ, JZ, nj,c,fl, currncat;
 SEXP RdimY, RdimX, Rdimo, Rdimb, RdimZ, Rdimu;
 double *betaX, *Yobs, *Ymiss, *mumiss, *omegadrawmiss, *betamiss, *betaobs, *omegaoo, *omegamo, *omegamm, *invomega, *invomega2, *help, *help2, *help3, *imp, *zi;
 double *sumzy, *incrzz, *incrzy, *mu, *mu2, *newbeta, *newomega, *sumzi, *yi, *invomega3, *help4, *help5, *help6, *missing, *fixomega,meanom,sdom, *resid, logLH, newlogLH,detom;
@@ -129,18 +128,19 @@ for (j=0; j<IY; j++) {
 r8mat_copy_new(IY, JY, REAL(Yimp2), imp);
 for (i=0;i<Ib*Jb;i++) REAL(betapost)[i]=0;
 for (i=0;i<JY*JY;i++) fixomega[i]=0;
-for (i=0;i<JY*JY;i++) newomega[i]=REAL(omega)[i];
 pos=ncon;
 for (i=0;i<ncat;i++) {
 	for (j=0;j<(INTEGER(Y_numcat)[i]-1);j++) {
 		for (k=0;k<(INTEGER(Y_numcat)[i]-1);k++) {
 			if (j==k) REAL(omega)[(pos+j)+Jo*(pos+k)]=1;
-			else REAL(omega)[(pos+j)+Jo*(pos+k)]=0;
+			else REAL(omega)[(pos+j)+Jo*(pos+k)]=0.5;
 			fixomega[(pos+j)+Jo*(pos+k)]=1;
 		}
 	}
 	pos=pos+INTEGER(Y_numcat)[i]-1;
 }
+for (i=0;i<JY*JY;i++) newomega[i]=REAL(omega)[i];
+
 GetRNGstate();
 
 /* Running ns iterations of Gibbs sampler*/
@@ -148,58 +148,92 @@ GetRNGstate();
 for (i=0;i<ns;i++) {
 
 	// Rejection Sampling
-
 	pos=ncon;
 	for (j=0;j<ncat;j++) {
+		currncat=INTEGER(Y_numcat)[j]-1;
 		for (t=0;t<IY;t++) {
 			if (!ISNAN(REAL(Y)[t+(ncon+j)*IY])) {
-				for (k=0;k<(INTEGER(Y_numcat)[j]-1);k++) betaX[k]=0;
+				for (k=0;k<currncat;k++) betaX[k]=0;
 				for (tt=0;tt<JX;tt++) {
-					for (k=0;k<(INTEGER(Y_numcat)[j]-1);k++) {
+					for (k=0;k<currncat;k++) {
 						betaX[k]=betaX[k]+REAL(beta)[tt+(k+pos)*Ib]*REAL(X)[t+tt*IX];
 					}
 				}
 				for (tt=0;tt<JZ;tt++) {
-					for (k=0;k<(INTEGER(Y_numcat)[j]-1);k++) {
+					for (k=0;k<currncat;k++) {
 						betaX[k]=betaX[k]+REAL(u)[(INTEGER(clus)[t])+nj*(tt+(k+pos)*JZ)]*REAL(Z)[t+tt*IZ];
 					}
 				}
 
 
-				for (k=0;k<pos;k++) help[k]=0;
+				for (k=0;k<(JY-currncat);k++) help[k]=0;
 				for (tt=0;tt<JX;tt++) {
 					for (k=0;k<pos;k++) {
 						help[k]=help[k]+REAL(beta)[tt+k*Ib]*REAL(X)[t+tt*IX];
 					}
+					for (k=(pos+currncat);k<JY;k++) {
+						help[k-currncat]=help[k-currncat]+REAL(beta)[tt+k*Ib]*REAL(X)[t+tt*IX];
+					}
 				}
 				for (tt=0;tt<JZ;tt++) {
 					for (k=0;k<pos;k++) {
 						help[k]=help[k]+REAL(u)[(INTEGER(clus)[t])+nj*(tt+k*JZ)]*REAL(Z)[t+tt*IZ];
 					}
+					for (k=(pos+currncat);k<JY;k++) {
+						help[k-currncat]=help[k-currncat]+REAL(u)[(INTEGER(clus)[t])+nj*(tt+k*JZ)]*REAL(Z)[t+tt*IZ];
+					}
 				}
 
 				for (k=0;k<pos;k++) {
 					help[k]=imp[t+k*IY]-help[k];
-					for (kk=0;kk<pos;kk++) help4[k+pos*kk]=REAL(omega)[k+JY*kk];
-					for (kk=0;kk<(INTEGER(Y_numcat)[j]-1);kk++) help5[k+pos*kk]=REAL(omega)[k+JY*(pos+kk)];
 				}
-				r8mat_pofac(pos,help4, help6,1);
-				r8mat_poinv(pos,help6, invomega);
-				for (jj=1;jj<pos;jj++) for (tt=0;tt<jj;tt++) invomega[jj+pos*tt]=invomega[tt+pos*jj];
-				r8mat_mm_new(pos,pos,(INTEGER(Y_numcat)[j]-1),invomega,help5, help2);
-				r8mat_mm_new(1,pos,(INTEGER(Y_numcat)[j]-1),help,help2, mumiss);
-				r8mat_add((INTEGER(Y_numcat)[j]-1),1,betaX,mumiss);
-				r8mat_mtm_new((INTEGER(Y_numcat)[j]-1),pos,(INTEGER(Y_numcat)[j]-1),help2,help5, omegadrawmiss);
-				r8mat_divide((INTEGER(Y_numcat)[j]-1),(INTEGER(Y_numcat)[j]-1),-1,omegadrawmiss);
-				for (k=0;k<(INTEGER(Y_numcat)[j]-1);k++) omegadrawmiss[k+k*(INTEGER(Y_numcat)[j]-1)]=omegadrawmiss[k+k*(INTEGER(Y_numcat)[j]-1)]+1;
-				r8mat_pofac((INTEGER(Y_numcat)[j]-1),omegadrawmiss, omegamm,2);
+				
+				for (k=(pos+currncat);k<JY;k++) {
+					help[k-currncat]=imp[t+k*IY]-help[k-currncat];
+				}
+				for (k=0;k<JY;k++) {
+					for (kk=0;kk<JY;kk++) {
+						if (((kk<pos)||(kk>(pos+currncat-1)))&&((k<pos)||(k>(pos+currncat-1)))) {
+							help4[countm]=REAL(omega)[kk+JY*k];
+							countm++;
+						}
+						else if (((kk<pos)||(kk>(pos+currncat-1)))&&((k>(pos-1))||(k<(pos+currncat)))) {
+							help5[counto]=REAL(omega)[kk+JY*k];
+							counto++;
+						}
+					}
+				}
+				countm=0;
+				counto=0;
+				r8mat_pofac((JY-currncat),help4, help6,1);
+				r8mat_poinv((JY-currncat),help6, invomega);
+			
+				for (jj=1;jj<(JY-currncat);jj++) for (tt=0;tt<jj;tt++) invomega[jj+(JY-currncat)*tt]=invomega[tt+(JY-currncat)*jj];
+					
+
+				r8mat_mm_new((JY-currncat),(JY-currncat),currncat,invomega,help5, help2);
+					
+				r8mat_mm_new(1,(JY-currncat),currncat,help,help2, mumiss);
+				r8mat_add(currncat,1,betaX,mumiss);
+				r8mat_mtm_new(currncat,(JY-currncat),currncat,help2,help5, omegadrawmiss);
+				r8mat_divide(currncat,currncat,-1,omegadrawmiss);
+				for (k=0;k<currncat;k++) {
+					omegadrawmiss[k+k*currncat]=omegadrawmiss[k+k*currncat]+1;
+					for (kk=0;kk<currncat;kk++) if (k!=kk) omegadrawmiss[k+kk*currncat]=omegadrawmiss[k+kk*currncat]+0.5;
+					
+				}
+				r8mat_pofac(currncat,omegadrawmiss, omegamm,2);
 				flag=0;
 				kk=0;
+
+
 				if (REAL(Y)[t+(ncon+j)*IY]==INTEGER(Y_numcat)[j]) {
 					while ((flag==0)&(kk<10000)) {
 						r8vec_multinormal_sample((INTEGER(Y_numcat)[j]-1), mumiss,omegamm, newbeta,mu4,fl);
 						maxim=maxvec((INTEGER(Y_numcat)[j]-1),newbeta);
 						if (maxim<0) {
+
+						
 							for (k=0;k<(INTEGER(Y_numcat)[j]-1);k++) imp[t+(k+pos)*IY]=newbeta[k];
 							flag=1;
 							indic++;
@@ -226,13 +260,15 @@ for (i=0;i<ns;i++) {
 		}
 		pos=pos+INTEGER(Y_numcat)[j]-1;
 	}
-//Rprintf("Fin qui tutto ok, iter= %d \n",i);
 
 	//Updating beta
+	
 
 	r8mat_pofac(JY,REAL(omega), help,3);
 	r8mat_poinv(JY,help, invomega);
+
 	for (jj=1;jj<Io;jj++) for (tt=0;tt<jj;tt++) invomega[jj+Io*tt]=invomega[tt+Io*jj];
+
 	for (j=0;j<JY*JY*JX*JX;j++) sumxi[j]=0;	
 	for (j=0;j<JY*JX;j++) sumxy[j]=0;
 	for (j=0;j<IY;j++) {
@@ -256,19 +292,22 @@ for (i=0;i<ns;i++) {
 		r8mat_mm_new(JY*JX,JY,1,help2,yi,incrxy);
 		r8mat_add(JY*JX,JY*JX,incrxx,sumxi);
 		r8mat_add(JY*JX,1,incrxy,sumxy);
+		
 	
 	}
 	
 	r8mat_pofac(JY * JX,sumxi,help3,4);
 	r8mat_poinv(JY * JX, help3,invomega2);
 	for (jj=1;jj<JX*JY;jj++) for (tt=0;tt<jj;tt++) invomega2[jj+JX*JY*tt]=invomega2[tt+JX*JY*jj];
+	
+
 	r8mat_mm_new(JY*JX,JY*JX,1,invomega2,sumxy,mu);
 	r8mat_pofac(JY * JX,invomega2,help3,5);
 	r8vec_multinormal_sample(JY*JX, mu,help3, REAL(beta),newbeta,fl);
 	r8mat_add(Ib,Jb,REAL(beta),REAL(betapost));
+	
 
 	//Updating random effects
-
 	for (c=0;c<nj;c++) {
 		r8mat_pofac(JY,REAL(omega),help,6);
 		r8mat_poinv(JY,help,invomega);
@@ -316,10 +355,8 @@ for (i=0;i<ns;i++) {
 		
 	}
 	r8mat_add(Iu,Ju,REAL(u),REAL(upost));
-
 	//Updating level 2 covariance matrix
 
-
 	for (j=0;j<JY*JY*JZ*JZ;j++) mu3[j]=0;
 	for (j=0;j<nj; j++) {
 		for (t=0;t<JY;t++) for (k=0;k<JZ;k++) uj[k+t*JZ]=REAL(u)[j+nj*(k+t*JZ)];
@@ -341,7 +378,6 @@ for (i=0;i<ns;i++) {
 	
 	r8mat_add(JY*JZ,JY*JZ,REAL(covu),REAL(covupost));
 
-
 	//Updating residuals
 
 	for (t=0;t<IY;t++) {
@@ -367,7 +403,7 @@ for (i=0;i<ns;i++) {
 		r8mat_mmt_new(1,JY,1,help5,help4,help6);
 		logLH=logLH+help6[0];
 	}
-	logLH=logLH*(-0.5)-IY*detom/2;
+	logLH=logLH*(-0.5)-IY*log(detom)/2;
 	for (j=0;j<JY;j++) {
 		for (k=0;k<JY;k++) {
 			if ((fixomega[j+JY*k]==0)&(j<=k)) {
@@ -383,7 +419,10 @@ for (i=0;i<ns;i++) {
 				while ((flag==0)&(kk<100)) {
 					newomega[j+JY*k]=r8_normal_sample(meanom,sdom,fl);
 					newomega[k+JY*j]=newomega[j+JY*k];
+			
 					flag=checkposdef(JY,newomega,help,help5);
+		
+
 					kk++;
 				}
 				r8mat_pofac(JY,newomega,help,13);
@@ -397,7 +436,7 @@ for (i=0;i<ns;i++) {
 					r8mat_mmt_new(1,JY,1,help5,help4,help6);
 					newlogLH=newlogLH+help6[0];
 				}
-				newlogLH=newlogLH*(-0.5)-IY*detom/2;
+				newlogLH=newlogLH*(-0.5)-IY*log(detom)/2;
 				if (((( double ) unif_rand ( ) )<exp(newlogLH-logLH))&(flag==1)) {
 					REAL(omega)[j+JY*k]=newomega[j+JY*k];
 					REAL(omega)[k+JY*j]=newomega[k+JY*j];
@@ -493,6 +532,7 @@ for (i=0;i<ns;i++) {
 	
 	}
 	if ((i+1)%10==0) Rprintf("Iteration %d completed\n",i+1);
+
 }
 for(i=0;i<IY;i++)  {
 	for(j=0;j<JY;j++)  {
