@@ -4,7 +4,6 @@
 #include <time.h>
 #include <string.h>
 #include "pdflib.h"
-#include "rnglib.h"
 #include "wishart.h"
 #include<R.h>
 #include<Rinternals.h>
@@ -12,7 +11,7 @@
 
 SEXP jomo1ranmix2(SEXP Y, SEXP Yimp, SEXP Yimp2, SEXP Yimpcat, SEXP X, SEXP Z, SEXP clus, SEXP beta, SEXP u, SEXP betapost, SEXP upost, SEXP omega, SEXP omegapost, SEXP covu, SEXP covupost,  SEXP nstep, SEXP Sp, SEXP Sup, SEXP Y_numcat, SEXP num_con, SEXP flagrng){
 int indic=0,i,j,k, IY,JY, IX, JX, Io, Jo, Ib, Jb, ns, nmiss=0,t, countm=0, counto=0, countmm=0, countmo=0, countoo=0, jj, tt, kk, ncon,ncat, pos,flag=0,nmaxx,h;
-int Iu, Ju, IZ, JZ, nj,c, fl;
+int Iu, Ju, IZ, JZ, nj,c, fl,currncat;
 SEXP RdimY, RdimX, Rdimo, Rdimb, RdimZ, Rdimu;
 double *betaX, *Yobs, *Ymiss, *mumiss, *omegadrawmiss, *betamiss, *betaobs, *omegaoo, *omegaom, *omegamo, *omegamm, *invomega, *invomega2, *help, *help2, *help3, *imp, *zi;
 double *sumzy, *incrzz, *incrzy, *mu, *mu2, *newbeta, *newomega, *sumzi, *yi, *invomega3, *help4, *help5, *help6, *missing, *fixomega, *resid;
@@ -131,18 +130,18 @@ GetRNGstate();
 r8mat_copy_new(IY, JY, REAL(Yimp2), imp);
 for (i=0;i<Ib*Jb;i++) REAL(betapost)[i]=0;
 for (i=0;i<JY*JY;i++) fixomega[i]=0;
-for (i=0;i<JY*JY;i++) newomega[i]=REAL(omega)[i];
 pos=ncon;
 for (i=0;i<ncat;i++) {
 	for (j=0;j<(INTEGER(Y_numcat)[i]-1);j++) {
 		for (k=0;k<(INTEGER(Y_numcat)[i]-1);k++) {
 			if (j==k) REAL(omega)[(pos+j)+Jo*(pos+k)]=1;
-			else REAL(omega)[(pos+j)+Jo*(pos+k)]=0;
+			else REAL(omega)[(pos+j)+Jo*(pos+k)]=0.5;
 			fixomega[(pos+j)+Jo*(pos+k)]=1;
 		}
 	}
 	pos=pos+INTEGER(Y_numcat)[i]-1;
 }
+for (i=0;i<JY*JY;i++) newomega[i]=REAL(omega)[i];
 
 
 /* Running ns iterations of Gibbs sampler*/
@@ -151,56 +150,96 @@ for (i=0;i<ns;i++) {
 
 	// Rejection Sampling
 
-	pos=ncon;
+		pos=ncon;
 	for (j=0;j<ncat;j++) {
+		currncat=INTEGER(Y_numcat)[j]-1;
+	
 		for (t=0;t<IY;t++) {
 			if (!ISNAN(REAL(Y)[t+(ncon+j)*IY])) {
-				for (k=0;k<(INTEGER(Y_numcat)[j]-1);k++) betaX[k]=0;
+				for (k=0;k<currncat;k++) betaX[k]=0;
 				for (tt=0;tt<JX;tt++) {
-					for (k=0;k<(INTEGER(Y_numcat)[j]-1);k++) {
+					for (k=0;k<currncat;k++) {
 						betaX[k]=betaX[k]+REAL(beta)[tt+(k+pos)*Ib]*REAL(X)[t+tt*IX];
 					}
 				}
 				for (tt=0;tt<JZ;tt++) {
-					for (k=0;k<(INTEGER(Y_numcat)[j]-1);k++) {
+					for (k=0;k<currncat;k++) {
 						betaX[k]=betaX[k]+REAL(u)[(INTEGER(clus)[t])+nj*(tt+(k+pos)*JZ)]*REAL(Z)[t+tt*IZ];
 					}
 				}
 
-				for (k=0;k<pos;k++) help[k]=0;
+
+				for (k=0;k<(JY-currncat);k++) help[k]=0;
 				for (tt=0;tt<JX;tt++) {
 					for (k=0;k<pos;k++) {
 						help[k]=help[k]+REAL(beta)[tt+k*Ib]*REAL(X)[t+tt*IX];
 					}
+					for (k=(pos+currncat);k<JY;k++) {
+						help[k-currncat]=help[k-currncat]+REAL(beta)[tt+k*Ib]*REAL(X)[t+tt*IX];
+					}
 				}
 				for (tt=0;tt<JZ;tt++) {
 					for (k=0;k<pos;k++) {
 						help[k]=help[k]+REAL(u)[(INTEGER(clus)[t])+nj*(tt+k*JZ)]*REAL(Z)[t+tt*IZ];
 					}
+					for (k=(pos+currncat);k<JY;k++) {
+						help[k-currncat]=help[k-currncat]+REAL(u)[(INTEGER(clus)[t])+nj*(tt+k*JZ)]*REAL(Z)[t+tt*IZ];
+					}
 				}
 
 				for (k=0;k<pos;k++) {
 					help[k]=imp[t+k*IY]-help[k];
-					for (kk=0;kk<pos;kk++) help4[k+pos*kk]=REAL(omega)[k+JY*kk];
-					for (kk=0;kk<(INTEGER(Y_numcat)[j]-1);kk++) help5[k+pos*kk]=REAL(omega)[k+JY*(pos+kk)];
 				}
-				r8mat_pofac(pos,help4, help6,1);
-				r8mat_poinv(pos,help6, invomega);
-				for (jj=1;jj<pos;jj++) for (tt=0;tt<jj;tt++) invomega[jj+pos*tt]=invomega[tt+pos*jj];
-				r8mat_mm_new(pos,pos,(INTEGER(Y_numcat)[j]-1),invomega,help5, help2);
-				r8mat_mm_new(1,pos,(INTEGER(Y_numcat)[j]-1),help,help2, mumiss);
-				r8mat_add((INTEGER(Y_numcat)[j]-1),1,betaX,mumiss);
-				r8mat_mtm_new((INTEGER(Y_numcat)[j]-1),pos,(INTEGER(Y_numcat)[j]-1),help2,help5, omegadrawmiss);
-				r8mat_divide((INTEGER(Y_numcat)[j]-1),(INTEGER(Y_numcat)[j]-1),-1,omegadrawmiss);
-				for (k=0;k<(INTEGER(Y_numcat)[j]-1);k++) omegadrawmiss[k+k*(INTEGER(Y_numcat)[j]-1)]=omegadrawmiss[k+k*(INTEGER(Y_numcat)[j]-1)]+1;
-				r8mat_pofac((INTEGER(Y_numcat)[j]-1),omegadrawmiss, omegamm,2);
+				
+				for (k=(pos+currncat);k<JY;k++) {
+					help[k-currncat]=imp[t+k*IY]-help[k-currncat];
+				}
+				for (k=0;k<JY;k++) {
+					for (kk=0;kk<JY;kk++) {
+						if (((kk<pos)||(kk>(pos+currncat-1)))&&((k<pos)||(k>(pos+currncat-1)))) {
+							help4[countm]=REAL(omega)[kk+JY*k];
+							countm++;
+						}
+						else if (((kk<pos)||(kk>(pos+currncat-1)))&&((k>(pos-1))||(k<(pos+currncat)))) {
+							help5[counto]=REAL(omega)[kk+JY*k];
+							counto++;
+						}
+					}
+				}
+				countm=0;
+				counto=0;
+									
+
+				r8mat_pofac((JY-currncat),help4, help6,1);
+				r8mat_poinv((JY-currncat),help6, invomega);
+			
+				for (jj=1;jj<(JY-currncat);jj++) for (tt=0;tt<jj;tt++) invomega[jj+(JY-currncat)*tt]=invomega[tt+(JY-currncat)*jj];
+					
+
+				r8mat_mm_new((JY-currncat),(JY-currncat),currncat,invomega,help5, help2);
+				
+
+				r8mat_mm_new(1,(JY-currncat),currncat,help,help2, mumiss);
+				r8mat_add(currncat,1,betaX,mumiss);
+				r8mat_mtm_new(currncat,(JY-currncat),currncat,help2,help5, omegadrawmiss);
+				r8mat_divide(currncat,currncat,-1,omegadrawmiss);
+				for (k=0;k<currncat;k++) {
+					omegadrawmiss[k+k*currncat]=omegadrawmiss[k+k*currncat]+1;
+					for (kk=0;kk<currncat;kk++) if (k!=kk) omegadrawmiss[k+kk*currncat]=omegadrawmiss[k+kk*currncat]+0.5;
+					
+				}
+				r8mat_pofac(currncat,omegadrawmiss, omegamm,2);
 				flag=0;
 				kk=0;
+	
+
 				if (REAL(Y)[t+(ncon+j)*IY]==INTEGER(Y_numcat)[j]) {
 					while ((flag==0)&(kk<10000)) {
-						r8vec_multinormal_sample((INTEGER(Y_numcat)[j]-1), mumiss,omegamm, newbeta,mu2,fl);
+						r8vec_multinormal_sample((INTEGER(Y_numcat)[j]-1), mumiss,omegamm, newbeta,mu4,fl);
 						maxim=maxvec((INTEGER(Y_numcat)[j]-1),newbeta);
 						if (maxim<0) {
+
+						
 							for (k=0;k<(INTEGER(Y_numcat)[j]-1);k++) imp[t+(k+pos)*IY]=newbeta[k];
 							flag=1;
 							indic++;
@@ -210,7 +249,7 @@ for (i=0;i<ns;i++) {
 				}
 				else {
 					while ((flag==0)&(kk<10000)) {
-						r8vec_multinormal_sample((INTEGER(Y_numcat)[j]-1), mumiss,omegamm, newbeta,mu2,fl);
+						r8vec_multinormal_sample((INTEGER(Y_numcat)[j]-1), mumiss,omegamm, newbeta,mu4,fl);
 						maxim=argmaxvec((INTEGER(Y_numcat)[j]-1),newbeta);
 						maxim2=maxvec((INTEGER(Y_numcat)[j]-1),newbeta);
 						if (((maxim+1)==REAL(Y)[t+(ncon+j)*IY])&(maxim2>0)) {
@@ -222,6 +261,7 @@ for (i=0;i<ns;i++) {
 					}
 				}
 				flag=0;
+
 			}
 		}
 		pos=pos+INTEGER(Y_numcat)[j]-1;
@@ -392,7 +432,7 @@ for (i=0;i<ns;i++) {
 			for (j=0;j<JY;j++) {
 				if (fixomega[k+JY*j]==1) {
 					if (k==j) invomega2[k+JY*j]=1;
-					else invomega2[k+JY*j]=0;
+					else invomega2[k+JY*j]=0.5;
 				}
 			}
 		}
